@@ -17,7 +17,7 @@ def _snps_fixup(snp_input, iid_if_none=None,count_A1=None):
 
 from pysnptools.snpreader.snpreader import SnpReader
 from pysnptools.snpreader.snpdata import SnpData
-from pysnptools.snpreader.bed import Bed
+from pysnptools.snpreader.bed import Bed, plink_chrom_map, reverse_plink_chrom_map
 from pysnptools.snpreader.ped import Ped
 from pysnptools.snpreader.dat import Dat
 from pysnptools.snpreader.snphdf5 import SnpHdf5
@@ -30,5 +30,3 @@ from pysnptools.snpreader._mergesids import _MergeSIDs
 from pysnptools.snpreader._mergeiids import _MergeIIDs
 from pysnptools.snpreader.snpgen import SnpGen
 from pysnptools.snpreader.distributedbed import DistributedBed, _Distributed1Bed
-from bed_reader import wrap_matrix_subset # To help other programs with backwards compatibility
-from pysnptools.snpreader import wrap_plink_parser # To help other programs with backwards compatibility
