@@ -206,6 +206,34 @@ class TestPySnpTools(unittest.TestCase):
         snpdata.val = 2 - snpdata.val
         self.c_reader(snpdata)
 
+    def test_bed_2021(self):
+        snpreader = Bed(self.currentFolder + "/examples/toydata.5chrom.bed",count_A1=True,
+                        fam_filename=self.currentFolder + "/examples/toydata.5chrom.2.fam",
+                        bim_filename=self.currentFolder + "/examples/toydata.5chrom.XYMT.bim")
+        snpdata = snpreader.read()
+
+        from pysnptools.snpreader import reverse_plink_chrom_map
+        import pysnptools.util as pstutil
+        pstutil.create_directory_if_necessary("tempdir/test_reverse_chrom_map.bed")
+        Bed.write("tempdir/test_reverse_chrom_map.bed",snpdata,count_A1=False,reverse_chrom_map=reverse_plink_chrom_map)
+
+
+        see_exception = False
+        try:
+            snpreader = Bed(self.currentFolder + "/examples/toydata.5chrom.bed",count_A1=True,
+                            bim_filename=self.currentFolder + "/examples/toydata.5chrom.bad.bim")
+            snpdata = snpreader.read()
+        except ValueError as e:
+            see_exception = True
+        assert see_exception
+
+        snpreader = Bed(self.currentFolder + "/examples/toydata.5chrom.bed",count_A1=True,
+                        bim_filename=self.currentFolder + "/examples/toydata.5chrom.bad.bim",
+                        chrom_map = {"B":1,"A":2,"D":3})
+        snpdata = snpreader.read()
+
+
+
     def test_bed_int8(self):
         snpreader = Bed(self.currentFolder + "/../tests/datasets/distributed_bed_test1_X.bed",count_A1=True)
         ref = snpreader.read()
@@ -452,12 +480,6 @@ class TestPySnpTools(unittest.TestCase):
             snps_C = np.array(snps, dtype=dtype, order="C")
             snp_s4 = Unit().standardize(snps_C)
 
-            snp_beta1 = Beta(1, 25).standardize(snps.copy(), force_python_only=True)
-            snps_F = np.array(snps, dtype=dtype, order="F")
-            snp_beta2 = Beta(1, 25).standardize(snps_F)
-            snps_C = np.array(snps, dtype=dtype, order="C")
-            snp_beta3 = Beta(1, 25).standardize(snps_C)
-
             self.assertEqual(snp_s1.shape[0], snp_s2.shape[0])
             self.assertEqual(snp_s1.shape[1], snp_s2.shape[1])
 
@@ -467,10 +489,17 @@ class TestPySnpTools(unittest.TestCase):
             self.assertEqual(snp_s1.shape[0], snp_s4.shape[0])
             self.assertEqual(snp_s1.shape[1], snp_s4.shape[1])
 
-            self.assertTrue(np.allclose(snp_s1, snp_s2, rtol=1e-05, atol=1e-05))
+            self.assertTrue(np.allclose(snp_s1, snp_s2, rtol=1e-05, atol=1e-05)) 
             self.assertTrue(np.allclose(snp_s1, snp_s3, rtol=1e-05, atol=1e-05))
             self.assertTrue(np.allclose(snp_s1, snp_s4, rtol=1e-05, atol=1e-05))
 
+
+            snp_beta1 = Beta(1, 25).standardize(snps.copy(), force_python_only=True)
+            snps_F = np.array(snps, dtype=dtype, order="F")
+            snp_beta2 = Beta(1, 25).standardize(snps_F)
+            snps_C = np.array(snps, dtype=dtype, order="C")
+            snp_beta3 = Beta(1, 25).standardize(snps_C)
+
             self.assertEqual(snp_beta1.shape[0], snp_beta2.shape[0])
             self.assertEqual(snp_beta1.shape[1], snp_beta2.shape[1])
             self.assertEqual(snp_beta1.shape[0], snp_beta3.shape[0])
@@ -483,7 +512,7 @@ class TestPySnpTools(unittest.TestCase):
         snpreader2 = Bed(self.currentFolder + "/examples/toydata.5chrom.bed",count_A1=False)
         self.load_and_standardize(snpreader2, snpreader2)
 
-        snpreader3 = Bed(self.currentFolder + "/examples/toydata.5chrom",count_A1=False)
+        snpreader3 = Bed(self.currentFolder + "/examples/toydata.5chrom",count_A1=False) 
         self.load_and_standardize(snpreader3, snpreader3)
 
 
@@ -762,7 +791,46 @@ class TestPySnpTools(unittest.TestCase):
 
         os.chdir(previous_wd)
 
+    def test_writes(self):
+        from pysnptools.snpreader import SnpData, SnpHdf5, SnpNpz, SnpMemMap
 
+        the_class_and_suffix_list = [(SnpMemMap,"memmap",None,None)]
+        
+        #===================================
+        #    Starting main function
+        #===================================
+        logging.info("starting 'test_writes'")
+        np.random.seed(0)
+        output_template = "tempdir/snpreader/writes.{0}.{1}"
+        create_directory_if_necessary(output_template.format(0,"npz"))
+        for row_count in [1]:
+            for col_count in [4]:
+                val = np.random.randint(0,4,size=(row_count,col_count))*1.0
+                val[val==3]=np.NaN
+                row = [('0','0'),('1','1'),('2','2'),('3','3'),('4','4')][:row_count]
+                col = ['s0','s1','s2','s3','s4'][:col_count]
+                for is_none in [True]:
+                    row_prop = None
+                    col_prop = None
+                    snpdata = SnpData(iid=row,sid=col,val=val,pos=col_prop)
+                    for the_class,suffix,constructor,writer in the_class_and_suffix_list:
+                        constructor = (lambda filename: the_class(filename))
+                        writer = (lambda filename,_data: the_class.write(filename,_data))
+
+                        filename = output_template.format("debug",suffix)
+                        logging.info(filename)
+                        ret = writer(filename,snpdata)
+                        assert ret is not None
+                        for subsetter in [np.s_[::2,::3]]:
+                            reader = constructor(filename)
+                            _fortesting_JustCheckExists().input(reader)
+                            subreader = reader if subsetter is None else reader[subsetter[0],subsetter[1]]
+                            readdata = subreader.read(order='C')
+                        try:
+                            os.remove(filename)
+                        except:
+                            pass
+        logging.info("done with 'test_writes'")
 
     def test_writes(self):
         from pysnptools.snpreader import SnpData, SnpHdf5, SnpNpz, SnpMemMap
