@@ -9,6 +9,7 @@ from types import ModuleType
 import inspect
 import warnings
 from pysnptools.util.intrangeset import IntRangeSet
+from bed_reader import get_num_threads, subset_f64_f64, subset_f32_f64, subset_f32_f32
 
 def _testtest(data, iididx):
     return (data[0][iididx], data[1][iididx])
@@ -267,9 +268,9 @@ def intersect_ids(idslist):
     return indarr
 
 
-def sub_matrix(val, row_index_list, col_index_list, order="A", dtype=np.float64):
+def sub_matrix(val, row_index_list, col_index_list, order="A", dtype=np.float64, num_threads=None):
     """
-    Efficiently creates a sub-matrix from a 2-D ndarray.
+    Efficiently creates a sub-matrix from a 2-D or 3-D ndarray.
 
     :param val: The ndarray from which to copy.
     :type val: ndarray
@@ -284,6 +285,10 @@ def sub_matrix(val, row_index_list, col_index_list, order="A", dtype=np.float64)
     :type order: string or None
     :param dtype: {numpy.float64 (default), numpy.float32}, optional -- The data-type for sub-matrix created.
     :type dtype: data-type
+    :param num_threads: optional -- The number of threads with which to work. Defaults to all available
+        processors. Can also be set with these environment variables (listed in priority order):
+        'PST_NUM_THREADS', 'NUM_THREADS', 'MKL_NUM_THREADS'.
+    :type num_threads: None or int
 
     :rtype: ndarray
 
@@ -297,9 +302,8 @@ def sub_matrix(val, row_index_list, col_index_list, order="A", dtype=np.float64)
     >>> print(matrix[2,0] == submatrix[1,6]) #The row # 2 is now #1, the column #0 is now #6.
     True
 
-    Note: Behind the scenes, for performance, this function selects and then calls one of 16 C++ helper functions.
+    Note: Behind the scenes, for performance, this function calls a Rust helper function.
     """
-    from bed_reader import wrap_matrix_subset
 
     if order == "A":
         if val.flags["F_CONTIGUOUS"]:
@@ -318,234 +322,61 @@ def sub_matrix(val, row_index_list, col_index_list, order="A", dtype=np.float64)
     else:
         assert original_dimensions == 3, "Expect val dimensions of 2 or 3"
         iid_count, sid_count, val_shape = val.shape
-    sub_val = np.empty(
-        (len(row_index_list), len(col_index_list), val_shape),
-        dtype=dtype,
-        order=effective_order,
-    )
-    sub_val.fill(np.nan)  # !!!keep? Replace empty with zeros????
 
-    logging.debug("About to call cython matrixSubset")
-    if val.flags["F_CONTIGUOUS"]:
+    def create_sub_val(dtype):
+        return np.full(
+        (len(row_index_list), len(col_index_list), val_shape),
+        np.NAN,
+        dtype=dtype,
+        order=effective_order
+        )   
+
+    logging.debug("About to call Rust matrixSubset")
+    num_threads = get_num_threads(num_threads)
+    if val.flags["F_CONTIGUOUS"] or val.flags["C_CONTIGUOUS"]:
+        row_index_list = np.asarray(row_index_list,dtype=np.uintp)
+        col_index_list = np.asarray(col_index_list,dtype=np.uintp)
         if val.dtype == np.float64:
-            if dtype == np.float64:
-                if effective_order == "F":
-                    wrap_matrix_subset.matrixSubsetDoubleFToDoubleFAAA(
-                        val,
-                        iid_count,
-                        sid_count,
-                        val_shape,
-                        row_index_list,
-                        col_index_list,
-                        sub_val,
-                    )
-                elif effective_order == "C":
-                    wrap_matrix_subset.matrixSubsetDoubleFToDoubleCAAA(
-                        val,
-                        iid_count,
-                        sid_count,
-                        val_shape,
-                        row_index_list,
-                        col_index_list,
-                        sub_val,
-                    )
-                else:
-                    raise Exception(
-                        "order '{0}' not known, only 'F' and 'C'".format(
-                            effective_order
-                        )
-                    )
-            elif dtype == np.float32:
-                if effective_order == "F":
-                    wrap_matrix_subset.matrixSubsetDoubleFToSingleFAAA(
-                        val,
-                        iid_count,
-                        sid_count,
-                        val_shape,
-                        row_index_list,
-                        col_index_list,
-                        sub_val,
-                    )
-                elif effective_order == "C":
-                    wrap_matrix_subset.matrixSubsetDoubleFToSingleCAAA(
-                        val,
-                        iid_count,
-                        sid_count,
-                        val_shape,
-                        row_index_list,
-                        col_index_list,
-                        sub_val,
-                    )
-                else:
-                    raise Exception(
-                        "dtype '{0}' not known, only float64 and float32".format(dtype)
-                    )
-        elif val.dtype == np.float32:
-            if dtype == np.float64:
-                if effective_order == "F":
-                    wrap_matrix_subset.matrixSubsetSingleFToDoubleFAAA(
-                        val,
-                        iid_count,
-                        sid_count,
-                        val_shape,
-                        row_index_list,
-                        col_index_list,
-                        sub_val,
-                    )
-                elif effective_order == "C":
-                    wrap_matrix_subset.matrixSubsetSingleFToDoubleCAAA(
-                        val,
-                        iid_count,
-                        sid_count,
-                        val_shape,
-                        row_index_list,
-                        col_index_list,
-                        sub_val,
-                    )
-                else:
-                    raise Exception(
-                        "order '{0}' not known, only 'F' and 'C'".format(
-                            effective_order
-                        )
-                    )
-            elif dtype == np.float32:
-                if effective_order == "F":
-                    wrap_matrix_subset.matrixSubsetSingleFToSingleFAAA(
-                        val,
-                        iid_count,
-                        sid_count,
-                        val_shape,
-                        row_index_list,
-                        col_index_list,
-                        sub_val,
-                    )
-                elif effective_order == "C":
-                    wrap_matrix_subset.matrixSubsetSingleFToSingleCAAA(
-                        val,
-                        iid_count,
-                        sid_count,
-                        val_shape,
-                        row_index_list,
-                        col_index_list,
-                        sub_val,
-                    )
-                else:
-                    raise Exception(
-                        "dtype '{0}' not known, only float64 and float32".format(dtype)
-                    )
-        else:
-            raise Exception(
-                "input dtype '{0}' not known, only float64 and float32".format(
-                    val.dtype
+            sub_val = create_sub_val(np.float64)
+            subset_f64_f64(
+                    val,
+                    row_index_list,
+                    col_index_list,
+                    sub_val,
+                    num_threads,
                 )
-            )
-    elif val.flags["C_CONTIGUOUS"]:
-        if val.dtype == np.float64:
             if dtype == np.float64:
-                if effective_order == "F":
-                    wrap_matrix_subset.matrixSubsetDoubleCToDoubleFAAA(
-                        val,
-                        iid_count,
-                        sid_count,
-                        val_shape,
-                        row_index_list,
-                        col_index_list,
-                        sub_val,
-                    )
-                elif effective_order == "C":
-                    wrap_matrix_subset.matrixSubsetDoubleCToDoubleCAAA(
-                        val,
-                        iid_count,
-                        sid_count,
-                        val_shape,
-                        row_index_list,
-                        col_index_list,
-                        sub_val,
-                    )
-                else:
-                    raise Exception(
-                        "order '{0}' not known, only 'F' and 'C'".format(
-                            effective_order
-                        )
-                    )
+                pass
             elif dtype == np.float32:
-                if effective_order == "F":
-                    wrap_matrix_subset.matrixSubsetDoubleCToSingleFAAA(
-                        val,
-                        iid_count,
-                        sid_count,
-                        val_shape,
-                        row_index_list,
-                        col_index_list,
-                        sub_val,
-                    )
-                elif effective_order == "C":
-                    wrap_matrix_subset.matrixSubsetDoubleCToSingleCAAA(
-                        val,
-                        iid_count,
-                        sid_count,
-                        val_shape,
-                        row_index_list,
-                        col_index_list,
-                        sub_val,
-                    )
-                else:
-                    raise Exception(
-                        "dtype '{0}' not known, only float64 and float32".format(dtype)
-                    )
+                warnings.warn("Converting float64 to float32 can cause loss of information")
+                sub_val = sub_val.astype(dtype,order)
+            else:
+                raise Exception(
+                    "dtype '{0}' not known, only float64 and float32".format(dtype)
+                )
         elif val.dtype == np.float32:
             if dtype == np.float64:
-                if effective_order == "F":
-                    wrap_matrix_subset.matrixSubsetSingleCToDoubleFAAA(
+                sub_val = create_sub_val(np.float64)
+                subset_f32_f64(
                         val,
-                        iid_count,
-                        sid_count,
-                        val_shape,
                         row_index_list,
                         col_index_list,
                         sub_val,
-                    )
-                elif effective_order == "C":
-                    wrap_matrix_subset.matrixSubsetSingleCToDoubleCAAA(
-                        val,
-                        iid_count,
-                        sid_count,
-                        val_shape,
-                        row_index_list,
-                        col_index_list,
-                        sub_val,
-                    )
-                else:
-                    raise Exception(
-                        "order '{0}' not known, only 'F' and 'C'".format(
-                            effective_order
-                        )
+                        num_threads
                     )
             elif dtype == np.float32:
-                if effective_order == "F":
-                    wrap_matrix_subset.matrixSubsetSingleCToSingleFAAA(
+                sub_val = create_sub_val(np.float32)
+                subset_f32_f32(
                         val,
-                        iid_count,
-                        sid_count,
-                        val_shape,
                         row_index_list,
                         col_index_list,
                         sub_val,
-                    )
-                elif effective_order == "C":
-                    wrap_matrix_subset.matrixSubsetSingleCToSingleCAAA(
-                        val,
-                        iid_count,
-                        sid_count,
-                        val_shape,
-                        row_index_list,
-                        col_index_list,
-                        sub_val,
-                    )
-                else:
-                    raise Exception(
-                        "dtype '{0}' not known, only float64 and float32".format(dtype)
-                    )
+                        num_threads
+                    )  
+            else:
+                raise Exception(
+                    "dtype '{0}' not known, only float64 and float32".format(dtype)
+                )
         else:
             raise Exception(
                 "input dtype '{0}' not known, only float64 and float32".format(
@@ -558,7 +389,7 @@ def sub_matrix(val, row_index_list, col_index_list, order="A", dtype=np.float64)
     if original_dimensions == 2:
         assert sub_val.shape[2] == 1, "real assert"
         sub_val = sub_val.reshape(sub_val.shape[0], sub_val.shape[1])
-    logging.debug("Back from cython matrixSubset")
+    logging.debug("Back from Rust matrixSubset")
     return sub_val
 
 
@@ -601,7 +432,7 @@ def create_directory_if_necessary(name, isfile=True, robust=False):
                 except OSError:
                     if not os.path.isdir(directory_name):
                         time_to_sleep *= 1.1
-                        warnings.warn(
+                        warnings.warning(
                             "creating directory robust=True, try#{0},time={3} error: not valid path: '{1}'. (Working directory is '{2}'".format(
                                 i, directory_name, os.getcwd(), int(time_to_sleep)
                             )
@@ -857,7 +688,7 @@ def array_module(xp=None):
         except ModuleNotFoundError as e:
             global _warn_array_module_once
             if not _warn_array_module_once:
-                logging.warn(f"Using numpy. ({e})")
+                logging.warning(f"Using numpy. ({e})")
                 _warn_array_module_once = True
             return np
 
