@@ -8,7 +8,7 @@ from unittest.mock import patch
 def work_sequence_to_result_sequence(work_sequence):
     '''
     Does all the work in a sequence of work items and returns a sequence of results.
-    '''
+    ''' 
     for work in work_sequence:
         if callable(work):
             result = work()
@@ -399,8 +399,6 @@ class SubGen:
         else:
             raise StopIteration()
 
-    next = __next__ # for Python 2
-
 
 from pysnptools.util.mapreduce1.runner.runner import Runner
 from pysnptools.util.mapreduce1.runner.local import Local, _JustCheckExists
